<?php

/*
Plugin Name: Ama Search Bar
Plugin URI: http://amasearchbar.com
Description: Adds an attention-grabbing, Amazon-custom-search bar on your blog.
Author: George Katsoudas
Developer: A. Oliinyk (Pumka.net)
Version: 2.2.2
Author URI: http://www.georgekatsoudas.com
Tested up to: 4.0
*/

/**
* Version history:
* 2.2.2
*   workaround for YP9-5T1-917G: Chrome scrolling issues
* 2.2.1
*   adapted to new Amazon search API
* 2.2
*   added: Amazon.in support
* 2.1.7
*   fixed: Update button on post/page editor
* 2.1.6
*   fixed: fixed W7S-L3A-J7NU: dialog title had HTML encoded
* 2.1.5
*   fixed: wrong product unavailable state under certain conditions
* 2.1.4
*   force usage of PluginUpdateChecker v1.3
* 2.1.3
*   new update check logic for less server load
* 2.1.2
*   fixes
* 2.1.1
*   new update URL
* 2.1
*   added: "Show on Mobile Devices" option
* 2.0.6
*   typo
* 2.0.5
*   added: rel="nofollow" for affiliate links
* 2.0.4
*   replaced GeoIP library
* 2.0.3
*   fixed: avoid GeoIP library redeclaration
* 2.0.2
*   fixed: star rating scraping didn't work for integer numbers
* 2.0.1
*   fixed: result count limit
* 2.0
*   Toolbar and search results template revamp
*   Drop-down Amazon market field in the toolbar
*   Admin option to hide discount drop-down from the toolbar
*   Options page and additional fields for static text customization
*   Tracking ID support
*   Search button text color and background color settings
*   Auto select market by user IP
* 1.2.1
*   fixed 68R-Z8V-XEVA: category in search results caused error
* 1.2
*   added: amazon.es support
* 1.1.14
*   fixed NMB-R9X-7WM7: workaround for iconv extension missing
*   allow dots in associate IDs
* 1.1.13
*   force float, margin and padding styles for fields and labels
* 1.1.12
*   prefer Offer price over LowestOfferPrice
* 1.1.11
*   CSS fix
* 1.1.10
*   fixed: spaces lost after HTML minification in WP Total Cache
* 1.1.9
*   force text-align
* 1.1.8
*   unique CSS classes
* 1.1.7
*   IE compatibility fixes
* 1.1.6
*   fixed demo
* 1.1.5:
*   bugfixes and refreshed Amazon category list
* 1.1.4:
*   IE9 CSS exclusion
* 1.1.3:
*   minor fixes
* 1.1.2:
*   added: filtering out product with descriptions empty after processing
* 1.1.1:
*   fixed: post editor form validatio error
*   fixed: missing search results for no keyword
* 1.1:
*   added: support for amazon,ca and amazon.de
* 1.0.2:
*   fixed: style issues with Sad Fish theme
* 1.0.1:
*   added: skip products with no description available
* 1.0: Initial release
**/

if (!class_exists('Asd_WpPlugin')) {
    require_once dirname(__FILE__) . '/library/compatibility.php';

    class Asd_WpPlugin
    {
    //------------------------------------------------------------------------------
    // Static members
    //------------------------------------------------------------------------------
        const PLUGIN_BASE_URL = 'http://amasearchbar.com/';
        const AUTHOR_BASE_URL = 'http://www.georgekatsoudas.com/';

        /**
        * Version tag. Used for CSS & JS versioning.
        */
        const VERSION = '02.02.02';

        /**
        * Unique slug
        */
        const SLUG = 'ama_search_bar';

        /**
        * Product name for auto-update system
        */
        const PRODUCT_NAME = 'ama_search_bar';

        /**
        * Unique prefix
        */
        const PREFIX = 'asd';

        /**
        * User-friendly title
        */
        const TITLE = 'Ama Search Bar';

        /**
        * Menu item text
        */
        const MENU_LABEL = 'Ama Search Bar';

        /**
        * Required capability to access plug-in pages on a single site installation
        */
        const CAPABILITY = 'administrator';

        static protected $_instance;

        static protected $_pluginBaseUrl;
        static protected $_pluginPath;

        static protected $_toolbarParams;

        static protected $_updater;
        static protected $_updaterCronHook;
        const UPDATE_CHECK_UNTERVAL = 12;

        /**
        * Registers plug-in module
        */
        static public function init()
        {
            // Auto update
            require dirname(__FILE__) . '/plugin-updates/plugin-update-checker.php';
            self::$_updater = new PluginUpdateChecker_1_3(
                'http://wppluginupdate.com/check/' . self::PRODUCT_NAME . '.json',
                __FILE__,
                self::PRODUCT_NAME,
                0
            );
            self::$_updaterCronHook = 'auto_check_updates-' . self::PRODUCT_NAME;
            add_action(self::$_updaterCronHook, array(self::$_updater, 'checkForUpdates'));

            self::$_pluginBaseUrl = plugin_dir_url(__FILE__);
            self::$_pluginPath = dirname(__FILE__);

            $class = get_class();

            register_activation_hook(__FILE__, array($class, 'activate'));
            register_deactivation_hook(__FILE__, array($class, 'deactivate'));

            // Admin actions
            if (is_admin()) {
                add_action('admin_init', array($class, 'adminInit'));
                add_action('admin_init', array($class, 'autoCheckUpdates'));
                add_action('admin_menu', array($class, 'adminMenu'));
                add_action('wp_ajax_' . self::PREFIX, array($class, 'adminAjax'));
                add_action('admin_enqueue_scripts', array($class, 'adminEnqueueResources'));
                add_action('save_post', array($class, 'savePost'), 10, 2);

                // Front-end AJAX recognized as admin
                add_action('wp_ajax_nopriv_' . self::PREFIX, array($class, 'frontendAjax'));
            } else {
                // Frontend actions
                add_action('init', array($class, 'frontendInit'));
                add_action('wp_enqueue_scripts',  array($class, 'frontendEnqueueResources'));
            }

            self::addLibraryIncludePath();

            // Detect an update
            if (self::VERSION != get_option(self::PREFIX . '_version')) {
                self::loadApplication();
                Asd_Application::getService('Setup')->install();
            }
        }

        static public function activate()
        {
            if (version_compare(PHP_VERSION, '5.0.0', '<')) {
                // Display Error Message
                self::throwInstallError(sprintf('Sorry, but %s requires PHP 5.0 or newer. Your version is %s. Please, ask your web host to upgrade to PHP 5.0.',
                    self::TITLE, phpversion()));
            }

            // Requires Wordpress 3.0+
            global $wp_version;
            if (version_compare($wp_version, '3.3', '<' )) {
                self::throwInstallError(sprintf('Sorry, but %s requires Wordpress 3.3 or newer. Your version is %s. Please, upgrade to the latest version of Wordpress.',
                    self::TITLE, $wp_version));
            }

            self::loadApplication();
            Asd_Application::getService('Setup')->install();
        }

        static public function throwInstallError($message)
        {
            if(function_exists('deactivate_plugins')) {
                deactivate_plugins(plugin_basename(__FILE__), true);
            }

            trigger_error($message, E_USER_ERROR);
        }

        /**
        * Plug-in deactivation hook
        */
        static public function deactivate()
        {
        }

        static public function addLibraryIncludePath()
        {
            $includePaths = explode(PATH_SEPARATOR, get_include_path());
            $path = realpath(dirname(__FILE__) . '/library');
            if (!in_array($path, $includePaths)) {
                foreach ($includePaths as $key=>$item) {
                    if ('.' == $item) {
                        unset($includePaths[$key]);
                        break;
                    }
                }
                array_unshift($includePaths, $path);
                array_unshift($includePaths, '.');
                set_include_path(implode(PATH_SEPARATOR, $includePaths));
            }
        }

        static public function loadApplication()
        {
            self::addLibraryIncludePath();
            require_once(self::$_pluginPath . '/application/Application.php');
        }

        static public function postEditorButtons()
        {
            return self::getInstance()->dispatch__post_editor__buttons();
        }

        static public function getHomePath()
        {
             return trailingslashit((string) parse_url(site_url(), PHP_URL_PATH));
        }

        static public function frontendInit()
        {
        }

        static public function adminInit()
        {
        }

        static public function autoCheckUpdates()
        {
            $updater = self::$_updater;
            $state = $updater->getUpdateState();

            $shouldCheck = (empty($state)
                || !isset($state->lastCheck)
                || ((time() - $state->lastCheck) >= (self::UPDATE_CHECK_UNTERVAL * 3600)))
                && !wp_next_scheduled(self::$_updaterCronHook);

            if ($shouldCheck) {
                wp_schedule_single_event(time(), self::$_updaterCronHook);
                spawn_cron();
            }
        }

        /**
        * admin_menu hook
        */
        static public function adminMenu()
        {
            self::getInstance()
                ->_registerAdminPages()
                ->_registerMetaBoxes();

        }

        static public function adminAjax()
        {
            self::getInstance()->dispatchAjax();
        }

        static public function frontendAjax()
        {
            $_POST['controller'] = 'frontend-ajax';

            self::getInstance()->dispatchAjax();
        }

        static public function savePost($postId, $post)
        {
            // Ignore new posts, autosave and revisions:
            if (wp_is_post_autosave($postId)
                || wp_is_post_revision($postId)
                || 'auto-draft' == $post->post_status
                || 'trash' == $post->post_status
                || (isset($_POST['action']) && 'autosave' == $_POST['action'])
                || 'nav_menu_item' == $post->post_type
            ) {
                return;
            }
            if (isset($_POST[self::PREFIX . '_metabox'])
                && is_array($_POST[self::PREFIX . '_metabox'])
            ) {
                self::getInstance()->dispatch__metabox__handle_save_post($postId,
                    $_POST[self::PREFIX . '_metabox']);
            }
        }

        static public function adminEnqueueResources($hook)
        {
            switch ($hook) {
                // Plug-in admin page
                case 'toplevel_page_' . self::SLUG:
                // Options page
                case str_replace('_', '-', self::SLUG) . '_page_' . self::SLUG . '-options':
                // Post editor
                case 'post.php':
                case 'post-new.php':
                    wp_enqueue_script(self::SLUG . '-admin',
                        self::getPluginBaseUrl() . 'js/admin.js',
                        array('jquery', 'jquery-ui-core', 'jquery-ui-dialog', 'farbtastic'),
                        self::VERSION);

                    wp_enqueue_style(self::SLUG . '-admin',
                        self::getPluginBaseUrl() . 'css/admin.css',
                        array('farbtastic', 'wp-jquery-ui-dialog'),
                        self::VERSION);
                    break;

                default:
                    // Do not include common stuff for other pages
                    return;
            }

            // Common stuff goes here:
            wp_enqueue_script(self::SLUG . '-help',
                self::getPluginBaseUrl() . 'js/help.js',
                array('jquery'),
                self::VERSION);
            wp_enqueue_style(self::SLUG . '-help',
                self::getPluginBaseUrl() . 'css/help.css',
                array(),
                self::VERSION);
        }

        /**
        * wp_enqueue_scripts hook
        */
        static public function frontendEnqueueResources()
        {
            self::$_toolbarParams = self::_getToolbarParams();

            if (self::$_toolbarParams) {
                wp_enqueue_style(self::SLUG . '-toolbar',
                    self::getPluginBaseUrl() . 'css/toolbar.css',
                    array(),
                    self::VERSION);

                wp_enqueue_script('jquery-html5placeholder',
                    self::getPluginBaseUrl() . 'js/html5placeholder.jquery.min.js',
                    array('jquery'),
                    '1.0.1');
                wp_enqueue_script(self::SLUG . '-toolbar',
                    self::getPluginBaseUrl() . 'js/toolbar.js',
                    array('jquery', 'jquery-ui-autocomplete'),
                    self::VERSION);
                wp_localize_script(self::SLUG . '-toolbar', self::PREFIX, array(
                    'ajaxurl' => admin_url('admin-ajax.php'),
                    'debug' => current_user_can('administrator')
                ));

                add_action('wp_footer', array(get_class(), 'renderToolbar'));
            }
        }

        static protected function _getToolbarParams()
        {
            if (!get_option(self::PREFIX . '_enabled')) {
                return;
            }

            $pageTypes = get_option(self::PREFIX . '_page_types');
            if (!$pageTypes || !is_array($pageTypes)) {
                return;
            }

            if (is_home() && in_array('home', $pageTypes)) {
                return array(
                    'page_type' => 'home',
                );
            }

            $postTypes = array();
            foreach ($pageTypes as $type) {
                if ('node_' == substr($type, 0, 5)) {
                    $postTypes[] = substr($type, 5);
                }
            }

            if ($postTypes && is_singular($postTypes)) {
                $id = get_the_ID();
                if ('0' === get_post_meta($id, self::PREFIX . '_enabled', TRUE)) {
                    // Not enabled
                    return NULL;
                }

                return array(
                    'page_type' => 'node',
                    'id' => $id,
                );
            }

            return NULL;
        }

        /**
        * Singleton pattern
        *
        * @return Asd_WpPlugin Singleton instance
        */
        static public function getInstance()
        {
            if (!self::$_instance) {
                self::$_instance = new self;
            }

            return self::$_instance;
        }

        static public function getPluginBaseUrl()
        {
            return self::$_pluginBaseUrl;
        }

        static public function getPluginPath()
        {
            return self::$_pluginPath;
        }

        //
        // Special getters & setters
        //

        static public function getToolbarParams()
        {
            return self::$_toolbarParams;
        }

        //
        // Renderers
        //

        static public function renderToolbar()
        {
            self::getInstance()->dispatch__toolbar(self::$_toolbarParams);
        }

    //------------------------------------------------------------------------------
    // Object members
    //------------------------------------------------------------------------------

        /**
        * Register plug-in admin pages and menus
        */
        protected function _registerAdminPages()
        {
            add_menu_page(self::TITLE, self::MENU_LABEL,
                self::CAPABILITY,
                self::SLUG,
                array($this, 'dispatchAdmin'),
                self::getPluginBaseUrl() . 'images/icon-16.png');

            add_submenu_page(self::SLUG, self::TITLE,
                'Admin', self::CAPABILITY,
                self::SLUG,
                array($this, 'dispatchAdmin'));

            add_submenu_page(self::SLUG, self::TITLE,
                'Options', self::CAPABILITY,
                self::SLUG . '-options',
                array($this, 'dispatch__admin__options'));

            return $this;
        }

        /**
        * Register plug-in admin metaboxes
        */
        protected function _registerMetaBoxes()
        {
            if (get_option(self::PREFIX . '_enabled')) {
                $postTypes = get_post_types(array('show_ui' => TRUE, 'show_in_nav_menus' => TRUE));
                $enabledPageTypes = get_option(self::PREFIX . '_page_types');
                if (!is_array($enabledPageTypes)) {
                    $enabledPageTypes = NULL;

                }

                foreach($postTypes as $type) {
                    if (is_null($enabledPageTypes)
                        || in_array('node_' . $type, $enabledPageTypes)
                    ) {
                        add_meta_box(self::PREFIX . '_main',
                            self::TITLE, array($this, 'dispatch__metabox__index'), $type,
                            'side', 'high');
                    }
                }
            }

            return $this;
        }

        /**
        * Intercepts WP requests
        */
        public function __call($method, $arguments)
        {
            if ('dispatch__' == substr($method, 0, 10)) {
                $parts = explode('__', $method);
                $controller = @$parts[1];
                $action = @$parts[2];

                $controller = $this->_prepareMvcName($controller);
                $action = $this->_prepareMvcName($action);

                $this->dispatch($controller, $action, $arguments);
            }
            else {
                throw new Exception("Method {$method} not found.");
            }
        }

        protected function _prepareMvcName($name)
        {
            return str_replace('_', '-', $name);
        }

        /**
        * Dispatches user request to WP Module
        *
        * @param string $controller Controller name
        * @param string $action Action name
        */
        public function dispatch($controller='index', $action='index',
            $params = NULL)
        {
            self::addLibraryIncludePath();

            require_once(self::$_pluginPath . '/application/Application.php');
            return Asd_Application::dispatch($controller, $action, $params);
        }

        public function dispatchAdmin()
        {
            $controller = @$_GET['controller'];
            if (empty($controller)) {
                $controller = 'admin';
            }
            $action = @$_GET['action'];
            if (empty($action)) {
                $action = 'index';
            }

            $this->dispatch($controller, $action);
        }

        public function dispatchAjax()
        {
            $controller = @$_POST['controller'];
            if (empty($controller)) {
                $controller = 'index';
            }
            $action = @$_POST['controller_action'];
            if (empty($action)) {
                $action = 'index';
            }

            $this->dispatch($controller, $action);

            exit();
        }
    }

    //Register plug-in:
    Asd_WpPlugin::init();

} else {
    deactivate_plugins(plugin_basename(__FILE__));
    trigger_error('This plugin cannot be installed together with '
        . Asd_WpPlugin::TITLE, E_USER_ERROR);
    return;
}