<?php
require_once ASD_APPLICATION_PATH . '/forms/AdminAbstract.php';

class Asd_Form_Admin extends Asd_Form_AdminAnstract
{
    /**
    * @var Asd_Model_Amazon_FormHelper
    */
    protected $_helper;

    protected $_marketFields = array(
        'category',
        'subcategory',
    );

    protected $_credentialFields = array(
        'access_key',
        'secret_access_key',
    );

    protected $_markets;

    public function init()
    {
        $this->_helper = Asd_Application::getModel('Amazon_FormHelper')
            ->setForm($this);

        //
        // Amazon API credentials
        //

        $this->addElement('text', 'access_key', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Access Key ID',
        ));

        $this->addElement('password', 'secret_access_key', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Secret Access Key',
        ));

        //
        // Market fields
        //
        $this->_markets = Asd_Application::getModel('Amazon_Market')->getList();
        foreach ($this->_markets as $marketCode=>$marketLabel) {
            $this->addElement('text', "{$marketCode}_affiliate_id", array(
                'filters' => array(
                    'StripNewlines',
                    'StringTrim',
                ),
                'validators' => array(
                    array('Regex', TRUE,
                        array(
                            'pattern' => '~^[a-zA-Z0-9\-\.]+$~u',
                            'messages' => array(
                                'regexNotMatch' => "'%value%' contains invalid characters",
                            ),
                        ),
                    )
                ),
                'label' => "{$marketLabel} Associate ID",
                'help' => FALSE,
            ));

            $this->addElement('select', "{$marketCode}_category", array(
                'filters' => array(
                    'Null',
                ),
                'label' => "{$marketLabel} Category",
                'disabled' => TRUE,
                'help' => FALSE,
            ));

            $this->addElement('select', "{$marketCode}_subcategory", array(
                'filters' => array(
                    'Null',
                ),
                'label' => "{$marketLabel} Subcategory",
                'disabled' => TRUE,
                'help' => FALSE,
            ));
        }

        //
        // Post types
        //

        $postTypes = Asd_Application::getModel('PostType')->getList('name');
        $list = array();
        foreach ($postTypes as $key=>$label) {
            $list['node_' . $key] = strtolower($label);
        }
        $list['home'] = 'homepage';

        $this->addElement('multiCheckbox', 'page_types', array(
            'multiOptions' => $list,
            'label' => 'Enable Search Bar on',
        ));

        //
        // Product search
        //

        $this->addElement('select', 'market', array(
            'required' => TRUE,
            'filters' => array(
                'Null',
            ),
            'multiOptions' => $this->_markets,
            'label' => 'Default Amazon Market'
        ));

        //
        // Display settings
        //

        $this->addElement('checkbox', 'discount_visible', array(
            'label' => 'Show Discount Option',
        ));

        $this->addElement('select', 'toolbar_position', array(
            'label' => 'Show Search Bar',
            'multiOptions' => array(
                'top' => 'At the top of the page',
                'bottom' => 'At the bottom of the page',
            ),
        ));

        $this->addElement('checkbox', 'show_on_mobile_devices', array(
            'label' => 'Show on Mobile Devices',
        ));

        $this->addElement('text', 'toolbar_background_color', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                'StringToLower',
            ),
           'validators' => array(
                array('Regex', TRUE, array(
                    'pattern' => '~^#[a-fA-F0-9]{6}$~',
                    'messages' => array('regexNotMatch' => '%value% is not a valid HTML color'),
                )),
            ),
            'label' => 'Bar Background Color',
        ));

        $this->addElement('text', 'search_button_text', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Search Button Text',
        ));

        $this->addElement('text', 'search_button_background_color', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                'StringToLower',
            ),
           'validators' => array(
                array('Regex', TRUE, array(
                    'pattern' => '~^#[a-fA-F0-9]{6}$~',
                    'messages' => array('regexNotMatch' => '%value% is not a valid HTML color'),
                )),
            ),
            'label' => 'Search Button Color',
        ));

        $this->addElement('text', 'search_button_text_color', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                'StringToLower',
            ),
           'validators' => array(
                array('Regex', TRUE, array(
                    'pattern' => '~^#[a-fA-F0-9]{6}$~',
                    'messages' => array('regexNotMatch' => '%value% is not a valid HTML color'),
                )),
            ),
            'label' => 'Search Button Text Color',
        ));

        $this->addElement('text', 'buy_button_text', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Add-to-Cart Button Text',
        ));

        $this->addElement('text', 'buy_button_background_color', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                'StringToLower',
            ),
           'validators' => array(
                array('Regex', TRUE, array(
                    'pattern' => '~^#[a-fA-F0-9]{6}$~',
                    'messages' => array('regexNotMatch' => '%value% is not a valid HTML color'),
                )),
            ),
            'label' => 'Add-to-Cart Button Color',
        ));

        $this->addElement('text', 'buy_button_text_color', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                'StringToLower',
            ),
           'validators' => array(
                array('Regex', TRUE, array(
                    'pattern' => '~^#[a-fA-F0-9]{6}$~',
                    'messages' => array('regexNotMatch' => '%value% is not a valid HTML color'),
                )),
            ),
            'label' => 'Add-to-Cart Button Text Color',
        ));
    }

    /**
     * Load categories && subcategories list
     */
    public function loadCategories($data=NULL)
    {
        $this->_helper->loadCategories($data);

        return $this;
    }

    /**
     * Check data
     */
    public function isValidEx($data)
    {
        $result = $this->_helper->isValid($data);

        if ($result) {
            // Check affiliate_id for default market
            $elem = $this->getElement("{$this->getValue('market')}_affiliate_id");
            if ('' == $elem->getValue()) {
                $elem->addError('Value is required and cannot be empty');
                $result = FALSE;
            }
        }

        return $result;
    }

    //
    // Special methods
    //

    public function extractCredentials($data=NULL)
    {
        $result = array();

        foreach ($this->_credentialFields as $field) {
            $value = $this->_extractField($field, $data);
            if ('' == $value) {
                return NULL;
            }
            $result[$field] = $value;
        }

        return $result;
    }

    protected function _extractField($field, $data=NULL)
    {
        if (is_null($data)) {
            return $this->getValue($field);
        }
        if (isset($data[$field])) {
            return $data[$field];
        }

        return NULL;
    }

    public function prepare()
    {
        foreach (array_keys($this->_markets) as $marketCode) {
            if (!$this->_hasCredentials($marketCode)) {
                foreach ($this->_marketFields as $field) {
                    $this->getElement("{$marketCode}_{$field}")->setAttrib('disabled', TRUE);
                }
            }
        }
    }

    protected function _hasCredentials($marketCode)
    {
        foreach ($this->_credentialFields as $field) {
            if ('' == $this->getValue($field)) {
                return FALSE;
            }
        }

        if ('' == $this->getValue("{$marketCode}_affiliate_id")) {
            return FALSE;
        }

        return TRUE;
    }

    //
    // Parent class overrides
    //

    public function save()
    {
        // Set enabled flag
        Asd_Application::getModel('Options')->setValue('enabled',
            $this->getValue('page_types')? 1 : 0);

        return parent::save();
    }

    //
    // Getters and setters
    //

    public function getMarkets()
    {
        return $this->_markets;
    }

    public function getMarketCodes()
    {
        return array_keys($this->_markets);
    }
}