<?php
require_once ASD_APPLICATION_PATH . '/forms/MetaboxAbstract.php';

class Asd_Form_Metabox extends Asd_Form_MetaboxAbstract
{
    protected $_helper;

    protected $_markets;
    protected $_categoryFields;
    protected $_defaultCategoryValues;

     protected $_marketFields = array(
        'category',
        'subcategory',
    );

    public function init()
    {
        $this->_helper = Asd_Application::getModel('Amazon_FormHelper')
            ->setForm($this);

        $this->addElement('checkbox', 'enabled', array(
            'label' => 'Enable search bar',
        ));

        $this->addElement('checkbox', 'customize_categories', array(
            'label' => 'Custom Market, Category and Subcategory',
        ));

        //
        // Market
        //

        $this->_markets = Asd_Application::getModel('Amazon_Market')->getList(TRUE);

        $field = 'market';
        $this->_categoryFields[] = $field;
        $this->addElement('select', $field, array(
            'required' => TRUE,
            'filters' => array(
                'Null',
            ),
            'multiOptions' => $this->_markets,
            'label' => 'Amazon Market',
        ));

        //
        // Market fields
        //

        foreach ($this->_markets as $marketCode=>$marketLabel) {
            $field = "{$marketCode}_category";
            $this->_categoryFields[] = $field;
            $this->addElement('select', $field, array(
                'filters' => array(
                    'Null',
                ),
                'label' => "{$marketLabel} Category",
                'help' => FALSE,
            ));

            $field = "{$marketCode}_subcategory";
            $this->_categoryFields[] = $field;
            $this->addElement('select', $field, array(
                'filters' => array(
                    'Null',
                ),
                'label' => "{$marketLabel} Subcategory",
                'help' => FALSE,
            ));
        }
    }

    //
    // Common methods use the helper object
    //

    /**
     * Load categories && subcategories list
     */
    public function loadCategories($data=NULL)
    {
        $this->_helper->loadCategories($data);

        $this->prepare();

        return $this;
    }

    /**
     * Check data
     */
    public function isValidEx($data)
    {
        if (!isset($data['customize_categories'])
            || !$data['customize_categories']
        ) {
            // Reset market and categories
            foreach ($this->_categoryFields as $field) {
                unset($data[$field]);
            }
        }

        return $this->_helper->isValid($data);
    }

    //
    // Custom methods
    //

    protected function _onUpdate()
    {
        if (!$this->getValue('customize_categories')) {
            // Reset categories to default
            foreach ($this->_getDefaultCategoryValues() as $key=>$value) {
                $this->setDefault($key, $value);
            }
        } else {
            // Auto fix default market if not supported
            $this->setDefault('market',
                Asd_Application::getModel('Amazon_Market')
                    ->suggest($this->getValue('market')));
        }

        return $this;
    }

    public function prepare()
    {
        if (!$this->getValue('customize_categories')) {
            // Disable and do not require
            foreach ($this->_categoryFields as $field) {
                $this->getElement($field)
                    ->setAttrib('disabled', TRUE)
                    ->setRequired(FALSE);
            }
        } else {
            // Enable categories and require market
            foreach ($this->_categoryFields as $field) {
                $this->getElement($field)->setAttrib('disabled', FALSE);
            }
            $this->getElement('market')->setRequired(TRUE);
        }

        return $this;
    }

    //
    // Helpers
    //

    protected function _getDefaultCategoryValues()
    {
        $options = Asd_Application::getModel('Options');

        if (is_null($this->_defaultCategoryValues)) {
            foreach ($this->_categoryFields as $field) {
                $this->_defaultCategoryValues[$field] = $options->getValue($field);
            }
        }

        return $this->_defaultCategoryValues;
    }

    //
    // Overrides
    //

    public function load()
    {
        $meta = Asd_Application::getModel('PostMeta')
            ->setPostId($this->_postId);
        $version = $meta->getValue('version');
        if ('' == $version) {
            $marketCode = $meta->getValue('market', NULL);
            if ('' != $marketCode) {
                foreach($this->_marketFields as $field) {
                    $value =  $meta->getValue($field, NULL);
                    if ('' != $value) {
                        $meta->setValue("{$marketCode}_{$field}", $value);
                    }
                    $meta->unsetValue($field);
                }
            }
        }

        $meta->setValue('version', Asd_WpPlugin::VERSION);

        parent::load();

        $this->_onUpdate();

        return $this;
    }

    public function save()
    {
        parent::save();

        $meta = Asd_Application::getModel('PostMeta')
            ->setPostId($this->_postId);
        $meta->setValue('version', Asd_WpPlugin::VERSION);

        return $this;
    }

    public function setDefaults(array $data)
    {
        if (!isset($data['customize_categories'])) {
            $data['customize_categories'] = 0;
        }

        parent::setDefaults($data);
        $this->_onUpdate();

        return $this;
    }

    public function extractCredentials($data=NULL)
    {
        return NULL;
    }

    //
    // Getters & setters
    //

    public function getMarkets()
    {
        return $this->_markets;
    }

    public function getMarketCodes()
    {
        return array_keys($this->_markets);
    }
}