/**
* Ama Search Bar toolbar JS
*/
"use strict";

var asd_searchXhr = null;

jQuery(function($) {
    $.fn.setDisabled = function(value) {
        if (typeof this['prop'] === 'function') {
            this.prop('disabled', value);
        } else if (value) {
            this.attr('disabled', 'disabled');
        } else {
            this.removeAttr('disabled');
        }

        if (value) {
            this.addClass('disabled');
        } else {
            this.removeClass('disabled');
        }

        return this;
    };

    $.fn.resetCategorySelect = function() {
        var item = $('<option value="" />');
        item.html(this.attr('data-any-label'));
        this.empty()
            .append(item);

        return this;
    };

    $.fn.waitCategorySelect = function(){
        this.empty()
            .setDisabled(true)
            .append('<option>loading...</option>');

        return this;
    };

    //
    // Init variables
    //

    var toolbarPosition = asd_getToolbarPosition();

    //
    // Event handlers
    //

    $('#asd-toolbar-market').change(function() {
        var value = $(this).val();

        $('#asd-toolbar-category').waitCategorySelect();

        $('#asd-toolbar-subcategory')
            .resetCategorySelect()
            .setDisabled(true);

        $.ajax({
            type: 'POST',
            url: asd.ajaxurl,
            dataType: 'json',
            data: {
                action: 'asd',
                controller: 'frontend-ajax',
                controller_action: 'get-categories',
                params: {
                    market: $('#asd-toolbar-market').val()
                },
                block_params: $('#asd-toolbar-block_params').val()
            },
            context: {
                category: $('#asd-toolbar-category'),
                subcategory: $('#asd-toolbar-subcategory')
            },
            success: function(data) {
                this.category
                    .resetCategorySelect();
                for (var i in data.categories) {
                    var item = data.categories[i]
                    if ('function' == typeof item) {
                        continue;
                    }

                    var option = $('<option/>', {
                        value: item['id'],
                        text: item['label']
                    });
                    if ('undefined' != typeof(item.selected)) {
                        option.attr('selected', 'selected');
                    }
                    this.category.append(option);
                }
                this.category.setDisabled(false);

                this.subcategory
                    .resetCategorySelect();
                if ('undefined' != typeof(data.subcategories)) {
                    for (var i in data.subcategories) {
                        var item = data.subcategories[i]
                        if ('function' == typeof item) {
                            continue;
                        }

                        var option = $('<option/>', {
                            value: item['id'],
                            text: item['label']
                        });
                        if ('undefined' != typeof(item.selected)) {
                            option.attr('selected', 'selected');
                        }
                        this.subcategory.append(option);
                    }
                    this.subcategory.setDisabled(false);
                } else {
                    this.subcategory.setDisabled(true);
                }
            },
            error: function(xhr) {
                this.category.resetCategorySelect();
                handleAjaxError(xhr);
            },
            complete: function() {
            }
        });
    });

    $('#asd-toolbar-category').change(function() {
        var value = $(this).val();
        if ('0' == value) {
            $('#asd-toolbar-subcategory')
                .resetCategorySelect()
                .setDisabled(true);

            return;
        }

        $('#asd-toolbar-subcategory').waitCategorySelect();

        $.ajax({
            type: 'POST',
            url: asd.ajaxurl,
            dataType: 'json',
            data: {
                action: 'asd',
                controller: 'frontend-ajax',
                controller_action: 'get-categories',
                params: {
                    market: $('#asd-toolbar-market').val(),
                    parent: value
                }
            },
            context: $('#asd-toolbar-subcategory'),
            success: function(data) {
                this.resetCategorySelect();
                for (var i in data) {
                    var item = data[i]
                    if ('function' == typeof item) {
                        continue;
                    }

                    this.append(jQuery('<option/>', {
                        value: item['id'],
                        text: item['label']
                    }));
                }
            },
            error: function(xhr) {
                this.resetCategorySelect();
                handleAjaxError(xhr);
            },
            complete: function() {
                this.setDisabled(false);
            }
        });
    });

    $('#asd-toolbar-hide-icon').click(asd_hideToolbar);
    $('#asd-toolbar-show-icon').click(asd_showToolbar);

    $(window).resize(positionResults);

    $('#asd-toolbar form.asd-search-form').submit(function() {
        positionResults();

        $('#asd-toolbar-wrapper .ui-autocomplete').hide();

        $('#asd-results .asd-results-container .asd-close').hide();
        $('#asd-results .asd-ajax-progress').slideDown();
        //$('#asd-results .asd-results-container .asd-placeholder').prop('scrollTop', 0);
        $('#asd-results .asd-results-container').slideUp(function() {
            $('.asd-placeholder', this)
                .empty();
        });

        $('.asd-button-search', this).setDisabled(true);

        if (asd_searchXhr) {
            asd_searchXhr.abort();
        }

        asd_searchXhr = $.ajax({
            type: 'POST',
            url: asd.ajaxurl,
            dataType: 'html',
            data: {
                action: 'asd',
                controller: 'frontend-ajax',
                controller_action: 'search',
                params: $(this).serialize()
            },
            context: this,
            success: function(data) {
                $('#asd-results .asd-results-container .asd-placeholder')
                    .empty()
                    .append(data);

                $('#asd-results .asd-results-container').slideDown(function () {
                    positionResults();
                    $('#asd-results .asd-results-container .asd-close').show();
                });
                $('#asd-results .asd-results-container .asd-placeholder').scrollTop(0); // reset scrollbar position
                $('#asd-results .asd-disclaimer-container').hover(
                    function() {
                        $('#asd-results .asd-disclaimer').fadeIn();
                    },
                    function() {
                        setTimeout('jQuery("#asd-results .asd-disclaimer").fadeOut();', 1000);
                    }
                );
            },
            error: function(xhr) {
                handleAjaxError(xhr);
            },
            complete: function() {
                $('#asd-results .asd-ajax-progress').slideUp('fast');
                $('.asd-button-search', this).setDisabled(false);
                asd_searchXhr = null;
            }
        });

        return false;
    });

    $('#asd-results .asd-results-container .asd-close').click(function() {
        $('#asd-results .asd-results-container .asd-close').hide();
        $('#asd-results .asd-results-container').slideUp('fast', function() {
            $('#asd-results .asd-results-container .asd-close').show();
        });

        return false; //suppress default processing
    });

    //
    // Functions
    //

    /**
    * Comom AJAX error handler
    *
    * @param xhr
    */
    function handleAjaxError(xhr) {
        if (0 === xhr.status && 'abort' === xhr.statusText) {
            // Terminated
            return;
        }

        if (500 != xhr.status) {
            alert('AJAX connection error');
        } else if (null != xhr.getResponseHeader('X-Ajax-Error')) {
            alert(xhr.responseText);
        } else if (null != xhr.getResponseHeader('X-Ajax-Error-Unhandled')) {
            alert('Unexpected error encountered. Please notify site owner.');
        } else {
            alert('Unexpected server error. Please notify site owner.');
        }
    }

    /**
    * Amazon completion source callback
    *
    * @param request
    * @param response
    */
    function amazonCompletionSource(request, response)
    {
        var searchAlias = asd_toolbar.searchAliases.all;
        var category = $('#asd-toolbar-category').val();
        if ('' != category && category in asd_toolbar.searchAliases) {
            searchAlias = asd_toolbar.searchAliases[category];
        }

        $.ajax({
            type: 'GET',
            url: asd_toolbar.completion.url,
            dataType: 'jsonp',
            data: {
                'q': request.term,
                'mkt': asd_toolbar.completion.mkt,
                'search-alias': searchAlias
            },
            context: response,
            success: function(data) {
                this(data[1]);
            }
        });
    }

    function positionResults()
    {
        var prop = toolbarPosition;

        var element = $('#asd-results');
        var x = ($(window).width() - element.width()) / 2;
        var y = $('#asd-toolbar').height();
        element.css('left', x)
            .css(prop, y);

        $('.asd-results-container .asd-placeholder', element)
            .css('max-height', 0.65 * $(window).height());

        var close = $('.asd-results-container .asd-close-icon', element);
        close.css('left', x + element.width() - close.outerWidth())
            .css(prop, y + ('bottom' == toolbarPosition? element.height() - 16 : 0));

        var close = $('.asd-results-container .asd-close-link', element);
        close.css('left', x + element.width() - close.outerWidth())
            .css(prop, y + ('top' == toolbarPosition? element.height() - 16 : 0));
    }

    //
    // Init code
    //

    $('#asd-toolbar-query').autocomplete({
        source: amazonCompletionSource,
        appendTo: '#asd-toolbar-menu-placeholder',
        position: { my: 'left top', at: 'left bottom', collision: 'flip' }
    });

    positionResults();

    window.setTimeout("asd_showToolbar();", asd.debug? 0 : 3000);
});


var asd_toolbarShown = false;

function asd_showToolbar()
{
    var toolbarPosition = asd_getToolbarPosition();
    var prop = toolbarPosition;

    // toolbar
    var element = jQuery('#asd-toolbar');
    var animateOptions = new Object();
    animateOptions[prop] = 0;
    element.css(prop, -element.outerHeight())
        .show()
        .animate(animateOptions);

    if (!asd_toolbarShown) {
        jQuery('#asd-toolbar-query').placeholder({
            'color': '#AAA',
            'placeholderCSS': {
                'text-align': 'left',
                'font': '12px Verdana,serif',
                'cursor': 'default',
                'position': 'absolute',
                'display': 'block',
                'overfolw': 'hidden',
                'left': '5px',
                'top': '4px'
            }
        });
    };

    // show tab
    var element = jQuery('#asd-toolbar-show');
    if (element.is(':visible')) {
        var animateOptions = new Object();
        animateOptions[prop] = -element.outerHeight();
        element.animate(animateOptions, 400, function() {
            jQuery(this).hide();
        });
    }

    asd_toolbarShown = true;
}

function asd_hideToolbar()
{
    if (asd_searchXhr) {
        asd_searchXhr.abort();
        asd_searchXhr = null;
    }

    var toolbarPosition = asd_getToolbarPosition();
    var prop = toolbarPosition;

    // results
    var element = jQuery('#asd-results');
    if (element.height() > 0) {
        jQuery('#asd-results .asd-results-container .asd-close').hide();
        element.slideUp('fast', function() {
            jQuery('.asd-results-element', this).hide();
            jQuery(this).show();
            asd_hideToolbar();
            jQuery('#asd-results .asd-results-container .asd-close').show();
        });
        return;
    }

    // toolbar
    var element = jQuery('#asd-toolbar');
    var animateOptions = new Object();
    animateOptions[prop] = -element.outerHeight();
    element.animate(animateOptions, 400, function() {
        jQuery(this).hide();
    });

    // show tab
    var element = jQuery('#asd-toolbar-show');
    var animateOptions = new Object();
    animateOptions[prop] = 0;
    element.css(prop, -element.outerHeight())
        .show()
        .animate(animateOptions);
}

function asd_getToolbarPosition()
{
    return jQuery('#asd-toolbar-wrapper').hasClass('asd-toolbar-position-bottom')
        ? 'bottom'
        : 'top';
}