<?php
/*
Plugin Name: Covert Store Content
Description: Auto-blogging plugin that will fill your Covert Store Builder sites with content from Amazon, Commission Junction and eBay on autopilot. You can also use the Covert Store Content plugin to import a batch of posts for editing. 
Version: 1.5
Author: IM Wealth Builders Ltd
Author URI: http://imwealthbuilders.com/newplugins

Copyright 2013  IM Wealth Builders

*/

define('IMWB_CSTCP_VERSION', '1.5');
define('IMWB_CSTCP_FOLDER', 'covertstorecontent');
define('IMWB_CSTCP_PREFIX', 'imwb_');

/**
 * @author John Merrick
 */
class IMWB_Covert_Store_Content {
	/**
	 * This value will be used for 
	 * - the gettext textdomain 
	 * - the subfolder when adding scripts and styles
	 * - the handle for scripts and styles
	 * Please make sure that this value corresponds to the folder name where your plugin resides.
	 * @var string 
	 */
	 var $plugin_name = "";
	 var $plugin_url = "";
	 var $update;
	 var $options = Array();
	 var $all_modules = Array("Amazon" => "amazon", "Commission Junction" => "commissionjunction", "eBay" => "ebay");
	 var $available_modules = Array();
    var $num_keywords = 100;
    var $currency_symbols = Array('AUD' => '$',
                                 'BWP'  => 'Pu',
                                 'CAD'  => '$',
                                 'DKK'  => 'kr',
                                 'GBP'  => '£',
                                 'HKD'  => 'HK$',
                                 'EUR'  => '€',
                                 'INR'  => '₹',
                                 'NGN'  => '₦',
                                 'PHP'  => 'Php',
                                 'USD'  => '$',
                                 'ZAR'  => 'R',
                                 'ZWD'  => 'Z$',
                                 'JPY'  => '￥',
                                 'CNY'  => '￥',
                                 'CHF'  => 'CHF',
                                 'TWD'  => 'TW$');                                  
		
	/**
	 * The constructor is executed when the class is instatiated and the plugin gets loaded.
	 * @return void
	 */
	function IMWB_Covert_Store_Content() {
	
	   $this->plugin_name = IMWB_CSTCP_PREFIX.IMWB_CSTCP_FOLDER;
	
   	// Hack to avoid firing on Admin Login or Registration page
   	if ( in_array( $GLOBALS['pagenow'], array( 'wp-login.php', 'wp-register.php' ) ) )
   	   return;

   	$this->plugin_url = plugins_url(basename( dirname( __FILE__)));
   	
      $this->options = $this->get_admin_options();
     	$this->load_modules();
     	
      if(is_admin()) {
         // Get Update Information
      	$this->get_update_info();
      	add_action('admin_notices', array(&$this, 'admin_warnings'));
     		add_action('after_plugin_row', array(&$this, 'check_for_update'));
      	add_action('admin_menu', array(&$this,'init_admin_menu'));
		   add_action('admin_init', array(&$this,'init_admin'));
      }
	}
	
	/**
	 * Register the admin option page stylesheet and scripts
	 */
	function init_admin() {
	   
	   wp_register_style($this->plugin_name . '-stylesheet', $this->plugin_url.'/styles/admin_options_page.css');
	   wp_register_script($this->plugin_name . '-script', $this->plugin_url.'/scripts/admin_options_page.js');
			   
	}

   /**
    * Scan the modules folder and see which ones are available
    *
    */
   function load_modules() {

      foreach ($this->all_modules as $moduleName => $moduleFile) {
         if(@file_exists(WP_PLUGIN_DIR.'/'.IMWB_CSTCP_FOLDER."/modules/".$moduleFile.".php")) {
            @require_once(WP_PLUGIN_DIR.'/'.IMWB_CSTCP_FOLDER."/modules/".$moduleFile.".php");
            $moduleClass = "IMWB_Covert_Store_Content_".$moduleFile;
            $this->available_modules[$moduleName] = new $moduleClass($this);         
            
         }
      }
   }
	
	function admin_warnings() {

		 if(empty($this->options['keywords']) && !isset($_POST['submit']) ) {
				echo "<div id='update-nag' class='update-nag'><strong>Covert Store Content is almost ready</strong> ".sprintf(__('You must <a href="%1$s">enter your keywords</a> and configure the settings.'), "options-general.php?page=imwb_covertstorecontent-admin-menu")."</div>";
			}
	}
	
	//Returns an array of admin options
	function get_admin_options() {
		$adminOptions = array('keywords' => '',
		                      'posttype' => '0',
		                      'createtags' => 'N',
		                      'badtags' => 'and,here,this,that,then,make,what,which,where,when,does,with,while,your,other,will',
		                      'storeimg' => 'N',
		                      'addcomments' => 'N',
		                      'randcomments' => 'N',
		                      'lastkw' => 0,
		                      'amzaffid' => '',
		                      'amzpubkey' => '',
		                      'amzprikey' => '',
		                      'amzregion' => 'com',
		                      'amzincfeatures' => 'N',
		                      'cjdevkey' => '',
		                      'cjwebpid' => '',
		                      'ebayid' => '',
		                      'ebayappid' => '',
		                      'ebaysiteid' => '0',
		                      'facebookappid' => '',
		                      'facebookappsecret' => '',
		                      'facebookwallid' => '',
		                      'twitterconkey' => '',
		                      'twitterconsecret' => '',
		                      'twitteraccesstoken' => '',
		                      'twitteraccesstokensec' => ''
		                      );
			
		$devOptions = get_option($this->plugin_name."-AdminOptions");
		if (!empty($devOptions)) {
			foreach ($devOptions as $key => $option)
				$adminOptions[$key] = $option;
		}				
		update_option($this->plugin_name."-AdminOptions", $adminOptions);
		return $adminOptions;
	}
	
   /**
    * Loading the Admin side javascripts
    * /scripts/admin_options_page.js from the plugin directory. 
    * @return void
    */
	
   function load_admin_scripts() {
   	// Only enqueue scripts on frontend. 
   	wp_enqueue_script('jquery');
   	wp_enqueue_script($this->plugin_name . '-script');
   
   	wp_localize_script('imwb_csctp_admin_options_script', 'IMWB_CSTCP_Ajax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ), 'nonce' => wp_create_nonce( 'imwb_csctp_clear_cookie_nonce' ), 'plugin_url' => $this->plugin_url, 'wp_root' => $this->get_wp_root_path() ) );
   }
	
	/*
	 * Will only be called on our plugin admin page, enqueue our stylesheet here
	 */
	function load_admin_styles() {
        wp_enqueue_style($this->plugin_name . '-stylesheet');
	}
	
	/**
	 * Adds a new menu with the name "My Admin Menu" and let's anyone that
	 * can publish posts be able to use it. 
	 * @return void
	 */
	function init_admin_menu() {
	
   	$view_hook_name = add_menu_page( 'Covert Store Content Admin Menu',
   	    'Covert Store Content',
   	    'manage_options',
   	    $this->plugin_name . '-admin-menu',
   	    array(&$this, 'load_menu_page'),
   	    '',
   	    318);
   	$this->views[$view_hook_name] = '1';
   	
   	add_action( 'admin_print_styles-' . $view_hook_name, array(&$this, 'load_admin_styles'));
   	add_action( 'admin_print_scripts-' . $view_hook_name, array(&$this, 'load_admin_scripts'));		
   	
   	$view_hook_name = add_submenu_page($this->plugin_name . '-admin-menu', 'Covert Store Content Settings', 'Settings', 'manage_options', $this->plugin_name . '-admin-menu', array(&$this, 'load_menu_page'));	
   	$this->views[$view_hook_name] = '1';		
   	add_action( 'admin_print_styles-' . $view_hook_name, array(&$this, 'load_admin_styles'));
   	add_action( 'admin_print_scripts-' . $view_hook_name, array(&$this, 'load_admin_scripts'));
   	
      foreach ($this->available_modules as $moduleName => $moduleClass) {
         $view_hook_name = $moduleClass->init_admin_menu();
         add_action( 'admin_print_styles-' . $view_hook_name, array(&$this, 'load_admin_styles'));
         add_action( 'admin_print_scripts-' . $view_hook_name, array(&$this, 'load_admin_scripts'));
      }
	}

	/**
	 * This function will be executed when the admin page is to be loaded
	 * @return void
	 */
	 function load_menu_page() {
	 
	    $this->currentBarNum = $this->views[current_filter()];
	 
	 	// Include the HTML from a separate file to keep the plugin class clean
	 	require "pages/admin_options_page.php";
	 }

	function get_wp_root_path()
	{
	    $path = parse_url(get_bloginfo('wpurl'), PHP_URL_PATH);
	    if(!isset($path))
	       return '/';
	    else
	  	    return $path;
	}
	
	// Function to get a list of the Blogs Categories and return a UL list of them
	function get_blog_categories($parentId = 0, $selected=array()) {
	   $retStr = '';
	
	if(!is_array($selected))
	   $selected = array();
	   
		$cats = get_categories('hide_empty=0&child_of='.$parentId.'&parent='.$parentId);
	   if(count($cats) > 0){
	      foreach($cats as $cat){
   	      $retStr.= "<option value='" . $cat->cat_ID . "'";
	         if( in_array($cat->cat_ID, $selected ) )
	         {
	         	$retStr .= " selected='selected' ";
	         }
	         $retStr .= " >" .($parentId > 0 ? '&nbsp;&nbsp;- ' :'').esc_html($cat->cat_name). "</option>";
	         
            $retStr .= $this->get_blog_categories($cat->cat_ID, $selected).'</li>';
	      }
	   }
	   
	   return $retStr;
	}
	
	function get_update_info() {
      // Include Update Class
      require_once('covertstorecontent-admin-update.php');
      
      // Initialize Update Object
      $this->update = new IMWB_Covert_SocialContentUpdate($this->plugin_name);	
	}
	
	function check_for_update($plugin)
	{			
      if(strpos($plugin, IMWB_CSTCP_FOLDER.'/') !== false) {
      	if(!$this->update->getUpdateInfo(1))
      	   echo '<tr class="plugin-update-tr"><td colspan="3" class="plugin-update"><div class="update-message">Warning: Could Not Connect To Covert Store Content Update Server</td></tr>';
      }			
	}		
	
	function get_next_keyword() {
	   $keywords = preg_split("/\n+/", stripslashes($this->options['keywords']));

	   if($this->options['lastkw'] >= count($keywords))
	      $this->options['lastkw'] = 0;
	      
	   $keyword = $keywords[$this->options['lastkw']];
	   
	   $this->options['lastkw']++;
	   update_option($this->plugin_name."-AdminOptions", $this->options);

	   return $keyword;
	}
	
	function get_post_categories($catIds, $catSelType, $keyword = null) {
	
	   $categories = explode(',', $catIds);

      if($catSelType == 'R')
   	   $post_categories = Array($categories[rand(0, count($categories)-1)]);
      else if($catSelType == 'M')
         $post_categories = $this->get_category_by_keyword($keyword);
   	else 
   	   $post_categories = $categories;
   	   
	   return $post_categories;
	}
	
	function get_category_by_keyword($keyword) {
	
      $keyword = str_replace(array('"', "'"),'', $keyword );
	   // If no keyword then 
	   if(empty($keyword))
	      return '';
	      
	   $categories = get_categories(array('hide_empty'=>0));
	   
	   foreach($categories as $category) {
	   
	      if (strcasecmp($keyword, $category->name) == 0) 
	         return Array($category->cat_ID);

         if (strcasecmp($keyword, $category->slug) == 0) 
            return Array($category->cat_ID);
      }
      
      // Not found so create it
      return Array(wp_create_category($keyword));
	}
	
	function get_tags($content) {
	

   	$contentTags = array();				
   	$badtags =  explode(',', htmlspecialchars(stripslashes($this->options['badtags'])));
  		$badchars = array(",", ":", "(", ")", "]", "[", "?", "!", ";", "-", '"');
  		
  		$content = str_replace($badchars, "", $content);
  		
     	$tags = explode(' ', $content);
  			
   	foreach ($tags as $key => $value) {
   		if (strlen($value) > 3) {
   			if (!in_array(strtolower($value), $badtags)) {
   				$contentTags[] = $value;
   			}
   		}
   	}

   	return $contentTags;
   }
   
   /**
    * Stores the passed in image URL in the media library and set the posts featured image
    *
    */
   function set_featured_image($postId, $imgUrl) {
   
      $response = wp_remote_get($imgUrl);
    
      if( is_wp_error( $response ) ) 
         return false;
           
      $filename = urldecode(basename( $imgUrl ));
      $filename = explode('?', $filename);
      
      $filename = sanitize_file_name($filename[0]);
      $mimeType = $response['headers']['content-type'];
              
      if(empty($filename)) {
       $filename = 'img_'.uniqid();
      }             
      
      // Check for file extentions
      $fileExt = pathinfo($filename, PATHINFO_EXTENSION);
             
      if(empty($fileExt)) {
         switch($mimeType) {
            case 'image/gif': $filename = $filename.".gif";
                            break;
            case 'image/png': $filename = $filename.".png";
                            break;
            case 'image/jpeg':
            case 'image/jpeg':
            case 'image/pjpeg':$filename = $filename.".jpg";
                            break;
         }
      }
      
   //   echo "Filename to Use: ".$filename."<br>";             
      $upload = wp_upload_bits( $filename, null, $response['body'] );
      $filename = $upload['file'];
      
      $wp_filetype = wp_check_filetype(basename($filename), null );
      $wp_upload_dir = wp_upload_dir();
      $attachment = array(
        'guid' => $wp_upload_dir['url'] . '/' . basename( $filename ), 
        'post_mime_type' => $wp_filetype['type'],
        'post_title' => preg_replace('/\.[^.]+$/', '', basename($filename)),
        'post_content' => '',
        'post_status' => 'inherit'
      );
      $attach_id = wp_insert_attachment( $attachment, $filename, $postId );
      // you must first include the image.php file
      // for the function wp_generate_attachment_metadata() to work
      require_once(ABSPATH . 'wp-admin/includes/image.php');
      $attach_data = wp_generate_attachment_metadata( $attach_id, $filename );
      wp_update_attachment_metadata( $attach_id, $attach_data );
      
      set_post_thumbnail( $postId, $attach_id );
      
      return true;
   }
		
	function do_posts($runNow = false) {

      //Find the active modules
   	foreach ($this->available_modules as $moduleName => $moduleClass) { 
   	   $moduleClass->cron($runNow);
		}
		
		return true;
	}
	
	// Posts the passed message to the configured social networks 
	function post_to_social_networks($postId, $postMessage) {
   	if (!class_exists('IMWB_Facebook')) 
   	   require_once WP_PLUGIN_DIR.'/'.IMWB_CSTCP_FOLDER."/facebook-api/facebook.php";
   	   
   	if (!class_exists('tmhOAuth')) 
   	   require_once WP_PLUGIN_DIR.'/'.IMWB_CSTCP_FOLDER."/twitter-api/tmhOAuth.php";
	   
	   // Facebook
   	if(!empty($this->options['facebookappid']) && !empty($this->options['facebookappsecret'])) {
  	   
   	   $facebook = new IMWB_Facebook(array(
   	     'appId'  => $this->options['facebookappid'],
   	     'secret' => $this->options['facebookappsecret'],
   	   ));
   	      	   
   	   // Get User ID
   	   $fb_user = $facebook->getUser();
   	   $fb_user_token = $facebook->getAccessToken();
   	 
   	   if (!empty($fb_user) && !empty($fb_user_token)) {
   	   
   	      if(!empty($this->options['facebookwallid'])) 
   	         $fb_wall_id = $this->options['facebookwallid'];
   	      else 
     	         $fb_wall_id = $fb_user;
   	
   	      wp_remote_post("https://graph.facebook.com/".$fb_wall_id."/feed", array(
   	      	'method' => 'POST',
   	      	'timeout' => 45,
   	      	'redirection' => 5,
   	      	'httpversion' => '1.0',
   	      	'blocking' => true,
   	      	'headers' => array(),
   	      	'body' => array( 'access_token' => $fb_user_token, 
   	      	                 'message' => $postMessage,
   	      	                 'caption' => get_the_title($postId),
   	      	                 'link' => get_permalink($postId),
   	      	                 'description' => strip_tags(get_the_excerpt($postId)) ),
   	      	'cookies' => array()
   	          )
   	      );
   	      
         }
   	}
   	
   	// Twitter
   	if(!empty($this->options['twitterconkey']) && !empty($this->options['twitterconsecret'])
   	   && !empty($this->options['twitteraccesstoken']) && !empty($this->options['twitteraccesstokensec'])) {

      	$tmhOAuth = new tmhOAuth(array(
      	  'consumer_key' => $this->options['twitterconkey'],
      	  'consumer_secret' => $this->options['twitterconsecret'],
      	  'user_token' => $this->options['twitteraccesstoken'],
      	  'user_secret' => $this->options['twitteraccesstokensec'],
      	));
      	
      	$code = $tmhOAuth->request('POST', $tmhOAuth->url('1.1/statuses/update'), array(
      	  'status' => $postMessage.' '.get_permalink($postId)
      	));
      	
//      	echo "TR: ".$code." - ".$tmhOAuth->response['response']."<br>";
      }   
   }
	
}

/**
 * Register the plugin
 */
add_action("init", create_function('', 'new IMWB_Covert_Store_Content();'));

register_deactivation_hook(__FILE__, 'IMWB_Covert_Store_Content_deactivation');

function IMWB_Covert_Store_Content_deactivation() {
	wp_clear_scheduled_hook(IMWB_CSTCP_PREFIX.IMWB_CSTCP_FOLDER.'_cron_event');
}

/**
 * Setup the Cron Job
 */
 
function IMWB_Covert_Store_Content_Cron_Job() {
 	
 	$ccClass = new IMWB_Covert_Store_Content();
 	$ccClass->do_posts(false);
}
 
add_action(IMWB_CSTCP_PREFIX.IMWB_CSTCP_FOLDER.'_cron_event', 'IMWB_Covert_Store_Content_Cron_Job');

if (!wp_next_scheduled(IMWB_CSTCP_PREFIX.IMWB_CSTCP_FOLDER.'_cron_event')) {
	wp_schedule_event( current_time( 'timestamp' ), 'hourly', IMWB_CSTCP_PREFIX.IMWB_CSTCP_FOLDER.'_cron_event');
} 
