<?php

class IMWB_Covert_Store_Content_commissionjunction {

   var $super = null;
   var $module_name = "commissionjunction";
   var $plugin_name = "";
   var $plugin_url = "";
   var $num_feed_urls = 100;
   var $options = Array();
   
   /**
    * The constructor is executed when the class is instatiated and the plugin gets loaded.
    * @return void
    */
   function IMWB_Covert_Store_Content_commissionjunction(&$super) {
      $this->super = $super;
      $this->plugin_name = $super->plugin_name;
      $this->plugin_url = $super->plugin_url;

      add_action('wp_ajax_commissionjunction_options_action', array(&$this, 'ajax_commissionjunction_options_action_callback'));
      
      wp_register_script($this->plugin_name .'-'.$this->module_name.'-script', $this->plugin_url.'/scripts/admin_commissionjunction_options_page.js');
      
   }
   
   function get_admin_options($feednum) {
      $adminOptions = array('active' => 'N',
                          	 'feedname' => '',
                            'posthours' => '4',
                            'postcount' => '5',
                            'categories' => '',
                            'catselecttype' => 'R',
                            'minprice' => '',
                            'maxprice' => '',
                            'keyword' => '',
                            'advertiserids' => '',                             
                            'socshare' => 'N',
                            'socmessage' => '',
                            'lastcat' => 0,
                            'lastrun' => 0);
   		
   	$devOptions = get_option($this->plugin_name."-".$this->module_name."-".$feednum."-AdminOptions");
   	if (!empty($devOptions)) {
   		foreach ($devOptions as $key => $option)
   			$adminOptions[$key] = $option;
   	}				

   	return $adminOptions;
   }
   
   function is_active(){
   
      return $this->options['active'] == 'Y' ? true : false;
   }
   
   function is_ready_to_post($feednum) {
   
      $lastrun = $this->options['lastrun'];
      if($lastrun > 0 && ($this->options['lastrun']+($this->options['posthours']*3600)) >current_time('timestamp') )
            return false;
      
      $this->options['lastrun'] = current_time('timestamp');
               
      update_option($this->plugin_name."-".$this->module_name."-".$feednum."-AdminOptions", $this->options);   
      
      return true;
   }
   
   function load_admin_scripts() {
   	wp_enqueue_script($this->plugin_name .'-'.$this->module_name.'-script');
   }
   
   function init_admin_menu() {
      $view_hook_name = add_submenu_page($this->plugin_name . '-admin-menu', 'Covert Store Content Commission Junction Settings', 'Commission Junction', 'manage_options', $this->plugin_name . '-admin-menu-'.$this->module_name, array(&$this, 'load_options_page'));
      
      add_action( 'admin_print_scripts-' . $view_hook_name, array(&$this, 'load_admin_scripts'));	
      
      return  $view_hook_name;
   }
   
   function load_options_page() {   
   	// Include the HTML from a separate file to keep the plugin class clean
   	require "commissionjunction_options_page.php";
   }
   
   function cron($runNow) {
   
      for($feednum = 1; $feednum <= $this->num_feed_urls; $feednum++) {
         $this->options = get_option($this->plugin_name."-".$this->module_name."-".$feednum."-AdminOptions");
         if($this->options !== false) {
   
      	   if($this->is_active()) {
               // See if module ready to run
               if($runNow || $this->is_ready_to_post($feednum)) {
                              
                  $result = $this->do_posts($feednum, stripslashes($this->options['keyword']));
                  
                  if(is_array($result)) {
                     $message = $result["error"]["module"]." - Reason: ".$result["error"]["reason"]." Message: ".$result["error"]["message"];
                     update_option($this->plugin_name."-".$this->module_name."-errors", $message);  
                  }
                  else
                     update_option($this->plugin_name."-".$this->module_name."-errors", ''); 
               }
      	   }
         }
      }
   }
   
   function do_posts($feed_num, $keyword, $numPosts = 0, $category_input = null, $cats_to_post_to = null, $tags_input = null, $post_status = null, $socshare = null) {   
      global $wpdb;
      
      $return = null;
      $numCreated = 0;
      $keyword = trim($keyword);
      
      // Get the Feed Urls to use
      if(empty($feed_num)){
         $return["error"]["module"] = "Commission Junction Feed";
         $return["error"]["reason"] = "No Feed Selected";
         $return["error"]["message"] = "There was no feed selected in the settings";
         return $return;
      }	
      
      $feedOptions = get_option($this->plugin_name."-".$this->module_name."-".$feed_num."-AdminOptions");
      if($feedOptions === false) {
         $return["error"]["module"] = "Commission Junction Feed (".$feedOptions['feedname'].")";
         $return["error"]["reason"] = "No Feed Options";
         $return["error"]["message"] = "The selected feed has no options";
         return $return;
      }	     
      
      if($numPosts == 0)
         $numPosts = rand(1, $feedOptions['postcount']);

      $sharePost = false;   
      if(($socshare == 'Y' && $feedOptions['socshare'] == 'Y') || ($socshare == null && $feedOptions['socshare'] == 'Y')) {         
         $sharePost = true;
         $sharePostNum = rand(1, $numPosts);
      }         
         
      // Loop through the result pages until we create enough posts
      $page = 1;
      do {

         $resultxml = $this->cj_request($keyword, $this->super->options['cjdevkey'], $this->super->options['cjwebpid'], $feedOptions['advertiserids'], $page, $feedOptions['minprice'], $feedOptions['maxprice']);	
      
         if(!empty($resultxml["error"])) {return $resultxml;}
         
         if ($resultxml === False) {
         	$return["error"]["module"] = "Commission Junction ($keyword)";
         	$return["error"]["reason"] = "API Request fail";
         	$return["error"]["message"] = "API request could not be sent.";	
         	return $return;	
         }
         
         if (!isset($resultxml->products->product)) {
         	$return["error"]["module"] = "Commission Junction ($keyword)";
         	$return["error"]["reason"] = "API Request";
         	$return["error"]["message"] = "API returned no products.";	
         	return $return;	
         }
         
         foreach($resultxml->products->product as $item) {	
         
            // Get the title
            $post_title = (string)$item->name;
            
            // Build the unique id
            if(!empty($item->upc) && $item->upc != "") {
               $unique = $item->upc;
               $uniqueId = "upc";
            } 
            elseif(!empty($item->sku)) {
               $unique = $item->sku;
               $uniqueId = "sku";
            } 
            elseif(!empty($item->isbn)) {
               $unique = $item->isbn;
               $uniqueId = "isbn";
            }
            
            $post_adid = trim((string)$item->{"advertiser-id"});
            $post_adid.= '_'.$uniqueId;
            $post_adid.= '_'.$unique;
            
   			// Check for exisintg post         
   			$results = $wpdb->get_row( $wpdb->prepare("SELECT post_id FROM {$wpdb->postmeta} WHERE meta_key = %s AND meta_value = %s", 'imwb_cc_commissionjunction_adid', $post_adid) );
   			
   			if(count($results) > 0) 
   			   continue;
            
            // Get the description
            $desc = (string)$item->description;
            
            // Get the image
            $post_image = (string)$item->{"image-url"};
            
            // Get the price
            $post_price = (string)$item->price;
            $post_currency = $this->super->currency_symbols[(string)$item->currency];
            
            $post_link = (string)$item->{"buy-url"};
   			   		
            // get category
            if(empty($cats_to_post_to))
               $post_category = $this->super->get_post_categories(htmlspecialchars(stripslashes($feedOptions['categories'])),$feedOptions['catselecttype'], $keyword);
            else 
               $post_category = $this->super->get_post_categories($category_input ,$cats_to_post_to, $keyword);
      
            // Get The Tags
            if(empty($tags_input)) {
               $post_tags = array();	
               if($this->super->options['createtags'] == 'Y') {
                  $post_tags = $this->super->get_tags($post_title);
               }
            }
            else 
               $post_tags = $tags_input;

            // Save Post
            $post_content = wpautop($desc);
            $post_type = "post";
            $post_date = current_time('mysql');
            $post_date_gmt = current_time('mysql', 1);
            $post_author = 1;
            if(empty($post_status))
               $post_status = $this->super->options['posttype'] == '0' ? 'publish' : 'draft';

   //        echo "Title: $post_title<br>Content: $post_content<br>CatId:$post_category<br>AuthorId:$post_author<br><br>";

           	$post_data = compact('post_content','post_title','post_date','post_date_gmt','post_author','post_category', 'post_status', 'post_type');		
            $post_data['tags_input'] = implode(',', $post_tags);
            $post_data = add_magic_quotes($post_data);
            $postId = wp_insert_post($post_data);
            if (is_wp_error($postId)) {
               $return["error"]["module"] = "Commission Junction ($keyword)";
               $return["error"]["reason"] = "Error Inserting Post";
               $return["error"]["message"] = $postId->get_error_message();	
               return $return;
            }
            
            // Store the product ID so we don't post duplicates
            update_post_meta( $postId , 'imwb_cc_commissionjunction_adid', $post_adid );
            
            // Update Featured image
            if($post_image != "") 
               $this->super->set_featured_image($postId, $post_image);
   
            // Set CSB Metadata fields
            update_post_meta( $postId, 'imwb_zonpress_post_producturl', $post_link);
            update_post_meta( $postId, 'imwb_zonpress_post_price', $post_price);
            update_post_meta( $postId, 'imwb_zonpress_post_currency', $post_currency);
            update_post_meta( $postId, '_imwb_zonpress_post_imp', 0 );
            update_post_meta( $postId, '_imwb_zonpress_post_clicks', 0);
            update_post_meta( $postId, '_imwb_zonpress_post_ctr', 0);
            
            // Set other IMWB plugins/themes metadata
            update_post_meta( $postId, 'imwb_interest_post_link_url', $post_link);
            update_post_meta( $postId, 'imwb_interest_post_link_text', 'From Commission Junction');
            
            $numCreated++;   
            
            if($sharePost == true && $sharePostNum == $numCreated && $post_status == 'publish') {
               $this->super->post_to_social_networks($postId, $feedOptions['socmessage']);
            }
            
            if( $numCreated == $numPosts)
               break;
         }

         // Page Loop
         if( $numCreated == $numPosts)
            break;         
            
         $page++;
                     
      } while($page <= 5);   
      
      return $numCreated;
   }
   
   function cj_request($keyword, $devkey, $websitepid, $advertiserids, $page, $minprice, $maxprice) {
   	libxml_use_internal_errors(true);
   	
      $keyword = '&keywords='.urlencode(stripslashes($keyword));
      
      if(!empty($advertiserids))
         $advertisers = $advertiserids;
      else
         $advertisers = 'joined';
      
      $lowprice = "";   
      if(!empty($minprice)) 
         $lowprice = "&low-price=".$minprice;
         
      $highprice = "";   
      if(!empty($maxprice)) 
         $highprice = "&high-price=".$maxprice;         
         
   	$requestURL = "https://product-search.api.cj.com/v2/product-search?website-id=$websitepid&advertiser-ids=$advertisers$keyword&sort-by=Name&page-number=$page&records-per-page=50$lowprice$highprice"; // The request URL used by the API service
   	
//  echo $requestURL."<br><br>";
   	$user_agent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2; rv:10.0.1) Gecko/20100101 Firefox/10.0.1";
   	$headers = array( 'Authorization' => $devkey );
   	$request = new WP_Http;
   	$response = $request->request( $requestURL , array( 'method' => 'GET', 'headers' => $headers ) );
   	
   	//$response = wp_remote_get($request, array('user-agent' => $user_agent, 'Authorization' => $devkey));
      if( is_wp_error( $response ) ) {
  	      $return["error"]["module"] = "Commission Junction";
  	      $return["error"]["reason"] = "Error Connecting to Commission Junction API";
  	      $return["error"]["message"] = $response->get_error_message();	
  	      return $return;
     }	 	
//echo '<br><br>Response:<pre>';
//   print_r( $response['body'] );
//   echo '</pre>';
     $pxml = simplexml_load_string($response['body']);
     if ($pxml === False) {
   		$emessage = "Failed loading XML, errors returned: ";
   		foreach(libxml_get_errors() as $error) {
   			$emessage .= $error->message . ", ";
   		}	
   		libxml_clear_errors();
   		$return["error"]["module"] = "Commission Junction";
   		$return["error"]["reason"] = "XML Error";
   		$return["error"]["message"] = $emessage;	
   		return $return;			
   	} else {
   		return $pxml;
   	}
   } 
   
  // AJAX callbacks
   function ajax_commissionjunction_options_action_callback() {
   	global $wpdb; // this is how you get access to the database
   
   	$feedNum = intval( $_POST['num'] );
   	$feedAction = $_POST['cmd'];
   	
   	if($feedAction == 'del') {
      	delete_option($this->plugin_name."-".$this->module_name."-".$feedNum."-AdminOptions");
      	echo "";
      	die();
      }
      
      $this->options = $this->get_admin_options($feedNum);
         
      ob_start();
?>   

<input type="hidden" name="imwb_covertstorecontent_pro_commissionjunction_options[lastrun]" value="<?php echo $this->options['lastrun'] ?>">
<input type="hidden" name="commissionjunctionfeednum" value="<?php echo $feedNum; ?>">

<table class="widefat">
 <thead>
  <tr>
   <th scope="col" colspan="2">Commission Junction Feed Settings</th>
  </tr>
</thead>
   <tbody style="display: table-row-group;">
   <tr valign="top"><td width="350px">Name</td>
   	<td>
   		<input id="imwb_covertstorecontent_pro_commissionjunction_options_feedname" class="regular-text" type="text" name="imwb_covertstorecontent_pro_commissionjunction_options[feedname]" value="<?php echo htmlspecialchars(stripslashes($this->options['feedname'])) ?>" size="40"/>
   		 <label class="description" for="imwb_covertstorecontent_pro_commissionjunction_options[feedname]">Enter a unique name to identify this feed</label>
   	</td>
   </tr>

<tr><td colspan="2"><strong>Keyword Search</strong></td></tr>
<tr valign="top"><td width="350px">Select keywords to use</td>
<td>
<select size="10" style="width:25em; height:  200px;" name="imwb_covertstorecontent_pro_commissionjunction_options[keyword]" id="imwb_covertstorecontent_pro_commissionjunction_options_keywords" >
<?php 
$selectedKeyword = htmlspecialchars(stripslashes($this->options['keyword']));
$keywords = preg_split("/[\s]\n+/", htmlspecialchars(stripslashes($this->super->options['keywords'])));
foreach ($keywords as $key => $value) {
   echo "<option value='".esc_attr($value)."'";
   if($value == $selectedKeyword )
   	echo " selected='selected' ";
   echo ">$value</option>";
}
?>
</select>
</td>
</tr>

<tr valign="top"><td width="350px">Restrict search to products in the following price range<br><small>Leave blank for no price range search</small></td>
<td>
Minimum Price: <input id="imwb_covertstorecontent_pro_commissionjunction_options_minprice" type="text" name="imwb_covertstorecontent_pro_commissionjunction_options[minprice]" value="<?php echo $this->options['minprice'] ?>" size="10"/>
Maximum Price: <input id="imwb_covertstorecontent_pro_commissionjunction_options_maxprice"  type="text" name="imwb_covertstorecontent_pro_commissionjunction_options[maxprice]" value="<?php echo $this->options['maxprice'] ?>" size="10"/>
 <label class="description" for="imwb_covertstorecontent_pro_commissionjunction_options[minprice]">Enter whole numbers only. I.e. 5, 10, 20 etc</label>

</td>
</tr>
<tr valign="top"><td width="350px">Advertiser Id's<br><small>You may provide a list of one or more advertiser CIDs, separated by commas, to limit the results to a specific sub-set of merchants.</small></td>
	<td><input id="imwb_covertstorecontent_pro_commissionjunction_options[advertiserids]" type="text" name="imwb_covertstorecontent_pro_commissionjunction_options[advertiserids]" value="<?php echo $this->options['advertiserids'] ?>" size="33">
	</td>	  
</tr>
<tr><td colspan="2"><strong>Categories To Use</strong></td></tr>
<tr valign="top"><td width="350px">Select categories to post to</td>
<td>
<input id="imwb_covertstorecontent_pro_commissionjunction_options_catselecttype" type="radio" name="imwb_covertstorecontent_pro_commissionjunction_options[catselecttype]" value="M" <?php echo $this->options['catselecttype'] == "M" ? "checked='Yes'" : "" ?> /> Post to category that matches keyword. Create if no match. <br>
<input id="imwb_covertstorecontent_pro_commissionjunction_options_catselecttype" type="radio" name="imwb_covertstorecontent_pro_commissionjunction_options[catselecttype]" value="R" <?php echo $this->options['catselecttype'] == "R" ? "checked='Yes'" : "" ?> /> Post to one category picked at random from your selection below <br>
<input id="imwb_covertstorecontent_pro_commissionjunction_options_catselecttype" type="radio" name="imwb_covertstorecontent_pro_commissionjunction_options[catselecttype]" value="A" <?php echo $this->options['catselecttype'] == "A" ? "checked='Yes'" : "" ?> /> Post to all selected categories<br><br>
<select size="10"  multiple="multiple"  style="width:25em; height:auto;" name="imwb_covertstorecontent_pro_commissionjunction_options[categories][]" id="imwb_covertstorecontent_pro_commissionjunction_options_categories" >
<?php 
   $currentCategories = explode(",", htmlspecialchars(stripslashes($this->options['categories'])) );
   echo	$this->super->get_blog_categories(0, $currentCategories ); 
?>
</select>
<br>
<p><a href="javascript:void(0)" onclick="imwb_covertstorecontent_pro_selectAll(document.getElementById('imwb_covertstorecontent_pro_commissionjunction_options_categories'))"><?php _e('Select All','imwb_covertstorecontent'); ?></a> / <a href="javascript:void(0)" onclick="imwb_covertstorecontent_pro_unselectAll(document.getElementById('imwb_covertstorecontent_pro_commissionjunction_options_categories'))"><?php _e('Unselect All','imwb_covertstorecontent'); ?></a></p>
</td>
</tr>

<tr><td colspan="2"><strong>Auto Blogging</strong></td></tr>
<tr valign="top"><td width="350px">Set this active and start auto blogging</td>
<td>
<input id="imwb_covertstorecontent_pro_commissionjunction_options[active]" type="checkbox" name="imwb_covertstorecontent_pro_commissionjunction_options[active]" value="Y" <?php echo $this->options['active'] == "Y" ? "checked='Yes'" : "" ?> /></td>
</tr>
<tr valign="top"><td width="350px">Post Creation</h3></td>
<td>
Create between 1 and  <input id="imwb_covertstorecontent_pro_commissionjunction_options[postcount]" type="text" name="imwb_covertstorecontent_pro_commissionjunction_options[postcount]" value="<?php echo $this->options['postcount'] ?>" size="3"> posts every <input id="imwb_covertstorecontent_pro_commissionjunction_options[posthours]" type="text" name="imwb_covertstorecontent_pro_commissionjunction_options[posthours]" value="<?php echo $this->options['posthours'] ?>" size="3"> hours
</td>
</tr>
<tr valign="top"><td width="350px">Auto Post Message to Social Networks<br><small>One message will be posted per autoblogging session for a post chosen randomly, providing the posts are published and not draft</small></td>
<td>
<input id="imwb_covertstorecontent_pro_commissionjunction_options[socshare]" type="checkbox" name="imwb_covertstorecontent_pro_commissionjunction_options[socshare]" value="Y" <?php echo $this->options['socshare'] == "Y" ? "checked='Yes'" : "" ?> /></td>
</tr>
<tr valign="top"><td width="350px">Message</td>
<td><input id="imwb_covertstorecontent_pro_commissionjunction_options[socmessage]" type="text" name="imwb_covertstorecontent_pro_commissionjunction_options[socmessage]" value="<?php echo $this->options['socmessage'] ?>" size="90">
</td>
</tr>
<tr><td colspan="2"><div class="submit">
<input class="button-primary" type="submit" name="update_imwb_covertstorecontentcommissionjunctionSettings" value="<?php echo $feedAction == 'add' ? 'Create Feed' : 'Update Feed';?>" />
</div></td></tr>
</tbody>
</table>

<?php   

      $output = ob_get_contents();
      ob_end_clean();
      echo $output;

   	die(); // this is required to return a proper result
   
   }   
}


?>